function obj = addcontrols(obj, items, parent)

    if nargin < 3 || isempty(parent)
        hparent = obj.Window;
    elseif iscell(parent)
        hparent = getfield(obj.Controls, parent{:});
    else
        hparent = obj.Controls.(parent);
    end
    
    for n = 1:numel(items)
        item = items{n};

        % Required: control name, style, string and position.
        name = item{1};
        style = item{2};
        string = item{3};
        position = item{4};
        
        % Optional: additional control attributes.
        if numel(items) >= 5
            additional = item(5:end);
        else
            additional = {};
        end

        % Adjust control attributes.
        if any(strcmpi(style, {'edit', 'popupmenu', 'listbox'}))
            additionalControlStyle = {'BackgroundColor', [1 1 1]};
        else 
            additionalControlStyle = {};
        end
        
        % Add control.
        h = uicontrol(...
            'Parent', hparent, ...
            obj.defaultGlobalStyle{:}, ...
            obj.defaultControlStyle{:}, ...
            'Style', style, ...
            'String', string, ...
            'Position', position, ...
            additionalControlStyle{:},...
            additional{:});

        if iscell(name)
            obj.Controls = setfield(obj.Controls, name{:}, h);
        elseif ~isempty(name)
            obj.Controls.(name) = h;
        end
    end
end
